import {
  select,
  csv,
  scaleLinear,
  scaleTime,
  extent,
  axisLeft,
  axisBottom,
  line,
  max
} from 'd3';

//importing the properties from the d3
const svg = select('svg'); //svg provides options to draw  shapes such as Lines and Circles.

// creating width and height for the svg diagram
const width = +svg.attr('width');
const height = +svg.attr('height');

const render = data => { //To render a React element into a root DOM node
  const title = '  Taylor Swift listening trends on spotify in the year 2001 '; // giving title for my line chart 
  
  const xValue = d => +d.Year; //displaying the x values ie months of 2001 from january to december
  const xAxisLabel = 'Year'; // labeling the x axis as Month

  const yValue = d => d.Selling_Price; // displaying the y values ie from 0 to 70 hours 
  const yAxisLabel = 'Selling_Price'; // labeling the y axis as Hours of Listening
  
  const margin = { top: 60, right: 40, bottom: 88, left: 150 };// providing the margin for line chart using css properties
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;
  //container
  
  const xScale = scaleTime()
  //complete set of values
    .domain(extent(data, xValue)) //returns the minimum and maximum value in an array from the given array using natural order.
    .range([0,innerWidth]) //output bounds
    .nice(); // it will round the values out
  
  const yScale = scaleLinear() // 
    .domain([0, max(data, yValue )])
    .range([innerHeight,0])
    .nice();
  
  const g = svg.append('g')
    .attr('transform', `translate(${margin.left},${margin.top})`);//center g in svg
  
  const xAxis = axisBottom(xScale)
    .tickSize(-innerHeight) // to get axis position at bottom
    .tickPadding(20);
  
  const yAxis = axisLeft(yScale)
    .tickSize(-innerWidth) // to get axis position at left side
    .tickPadding(10);
  
  const yAxisG = g.append('g').call(yAxis); //It appends a 'g' element to the SVG. g element is used to group SVG shapes together
  	yAxisG.selectAll('.domain').remove();
  
  yAxisG.append('text') // for labeling the text
      .attr('class', 'axis-label')
      .attr('y', -100)
      .attr('x', -innerHeight / 4)
      .attr('fill', 'black')
      .attr('transform', `rotate(-90)`)
      .text(yAxisLabel);
  
  const xAxisG = g.append('g').call(xAxis)
    	.attr('transform', `translate(0,${innerHeight})`);
  
  xAxisG.select('.domain').remove();
  
  // for x axis labeling
  xAxisG.append('text') 
      .attr('class', 'axis-label')
      .attr('y', 70)
      .attr('x', innerWidth / 2)
      .attr('fill', 'black')
      .text(xAxisLabel);
  
  // it generrates horizantal and vertical lines for x and y axis
  const lineGenerator = line() 
  	.x(d => xScale(xValue(d)))
  	.y(d => yScale(yValue(d))); //coordinates in pixels
  
  // provides path to the line
  g.append('path')  
  	.attr('class', 'line_path')
  	.attr('d', lineGenerator(data))
    console.log(lineGenerator(data))
  
   // for labeling the title
  g.append('text')
      .attr('class', 'title')
      .attr('y', -20)
      .attr('x', innerWidth / 5)
      .text(title);
};

// getting the csv data uisng github url link for my (swift_trends.csv)data 
csv('https://gist.githubusercontent.com/Shivamaninaidu/8437ec09c24fa084630088b547570f7b/raw/95c633420631a445118f0ab105909068aafda3c7/fortuner.csv')
  .then(data => {
  
  	//console.log(data)
    data.forEach(d => {
    	d.Hours = +d.Hours;
      d.Date = new Date(d.Date)
    });
  console.log(data)
    render(data);
  });